/***************************************************************************
 *   Copyright (C) 2005 - 2006 by Christian Muehlhaeuser, Last.fm Ltd.            *
 *   chris@last.fm                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
 ***************************************************************************/

#include "settings.h"
#include "md5/md5.h"
#include "rtaudio/RtAudio.h"

Settings *Settings::s_instance = 0;


Settings::Settings()
    : QObject()
    , m_debug( false )
    , m_actAsProxy( false )
{
    s_instance = this;

    config = new QSettings( QSettings::IniFormat, QSettings::UserScope, "last.fm", "player", this );
    config->setFallbacksEnabled( false );

    qDebug( QString( "Settings filename: " + config->fileName() ).toLocal8Bit() );
}


QString
Settings::MD5Digest( const char *token )
{
    md5_state_t md5state;
    unsigned char md5pword[16];

    md5_init( &md5state );
    md5_append( &md5state, (unsigned const char *)token, (int)strlen( token ) );
    md5_finish( &md5state, md5pword );

    char tmp[33];
    strncpy( tmp, "\0", sizeof(tmp) );
    for ( int j = 0; j < 16; j++ )
    {
        char a[3];
        sprintf( a, "%02x", md5pword[j] );
        tmp[2*j] = a[0];
        tmp[2*j+1] = a[1];
    }

    return QString::fromAscii( tmp );
}


bool
Settings::resumePlayback()
{
    return ( config->value( "player/resumeplayback" ).toInt() == 1 );
}


void
Settings::setResumePlayback( bool enabled )
{
    config->setValue( "player/resumeplayback", enabled ? "1" : "0" );
}


QString
Settings::resumeStation()
{
    return config->value( "player/resumestation" ).toString();
}


void
Settings::setResumeStation( QString station )
{
    config->setValue( "player/resumestation", station );
}


int
Settings::volume( )
{
   return config->value( "player/volume", 50 ).toInt();
}


void
Settings::setVolume( int volume )
{
   config->setValue( "player/volume", volume );
}


int
Settings::positionX()
{
    return config->value( "player/posx", 0 ).toInt();
}


void
Settings::setPositionX( int x )
{
    config->setValue( "player/posx", x );
    config->sync();
}


int
Settings::positionY()
{
    return config->value( "player/posy", 0 ).toInt();
}


void
Settings::setPositionY( int y )
{
    config->setValue( "player/posy", y );
    config->sync();
}


int
Settings::soundSystem()
{
    return config->value( "player/soundsystem", 0 ).toInt();
}


void
Settings::setSoundSystem( int system )
{
    config->setValue( "player/soundsystem", system );
    config->sync();
}


int
Settings::soundCard()
{
    return config->value( "player/soundcard", 0 ).toInt();
}


void
Settings::setSoundCard( int card )
{
    if ( card < 0 )
        card = 0;

    config->setValue( "player/soundcard", card );
    config->sync();
}


QString
Settings::username()
{
    return config->value( "player/username" ).toString();
}


QString
Settings::password()
{
    QString password = config->value( "player/password" ).toString();
    if ( password.length() < 32 && !password.isEmpty() )
    {
        setPassword( password );
        password = MD5Digest( password.toAscii() );
    }

    return password;
}


QString
Settings::proxyUsername()
{
    return config->value( "player/proxyusername" ).toString();
}


QString
Settings::proxyPassword()
{
    return config->value( "player/proxypassword" ).toString();
}


QString
Settings::proxyHost()
{
    return config->value( "player/proxyhost" ).toString();
}


int
Settings::proxyPort()
{
    return config->value( "player/proxyport" ).toInt();
}


bool
Settings::proxyUsage()
{
    return ( config->value( "player/proxyusage" ).toInt() == 1 );
}


void
Settings::setUsername( QString username )
{
    config->setValue( "player/username", username );
}


void
Settings::setPassword( QString password )
{
    if ( !password.isEmpty() && password != "********" )
    {
        password = MD5Digest( password.toAscii() );
        config->setValue( "player/password", password );
    }
}


void
Settings::setProxyUsername( QString username )
{
    config->setValue( "player/proxyusername", username );
}


void
Settings::setProxyPassword( QString password )
{
    config->setValue( "player/proxypassword", password );
}


void
Settings::setProxyHost( QString host )
{
    config->setValue( "player/proxyhost", host );
}


void
Settings::setProxyPort( int port )
{
    config->setValue( "player/proxyport", port );
}


void
Settings::setProxyUsage( bool enabled )
{
    config->setValue( "player/proxyusage", enabled ? "1" : "0" );
}


QString
Settings::browser()
{
    return config->value( "player/browser" ).toString();
}


void
Settings::setBrowser( QString browser )
{
    config->setValue( "player/browser", browser );
}


int
Settings::stationCounter( QString url )
{
    return config->value( "history/" + url ).toInt();
}


QString
Settings::stationName( QString url )
{
    QString name = config->value( "stations/" + url ).toString();
    if ( !name.isEmpty() )
    {
        QStringList parts = name.split( ",", QString::SkipEmptyParts );
        name = "";

        for ( int i = 0; i < parts.count() && i < 5; i++ )
        {
            if ( i ) name += ", ";
            name += parts.at( i );
        }
    }

    return name;
}


void
Settings::incrementStationCounter( QString url, QString stationName )
{
    if ( stationName != 0 )
    {
        config->setValue( "stations/" + url, stationName );
    }

    config->setValue( "history/" + url, stationCounter( url ) + 1 );
    config->sync();
}


void
Settings::deleteStationCounter( QString url )
{
    config->remove( "history/" + url );
    config->sync();
}


void
Settings::resetStationCounter( QString url )
{
    config->setValue( "history/" + url, 0 );
    config->sync();
}


void
Settings::clearStationCounters()
{
    config->remove( "history" );
    config->sync();
}


QStringList
Settings::stationItems()
{
    config->sync();

    config->beginGroup( "history" );
    QStringList items = config->allKeys();
    config->endGroup();

    return items;
}


void
Settings::save( bool restart )
{
    config->sync();

    if ( restart )
        emit configChanged();
}


void
Settings::startBrowser( QString url )
{
    #ifdef Q_WS_X11
    if ( browser().isEmpty() )
        QMessageBox::information( 0, "Attention", "Please configure a Browser in the Settings-Dialog!", QMessageBox::Ok );
    #endif

    if ( !url.isEmpty() && !browser().isEmpty() )
    {
        BrowserThread thread;
        thread.setUrl( url );
        thread.start();
    }
}


void
BrowserThread::run()
{
    qDebug( "Starting Browser!" );

    #ifdef WIN32
    WCHAR val[1024];
    memset( val, 0, 1024 );
    MultiByteToWideChar( CP_UTF8, 0, m_url.toAscii(), m_url.length(), val, 1024 );
    ShellExecute( 0, 0, val, NULL, NULL, SW_SHOW );
    #endif

    #ifdef Q_WS_MAC
    ICInstance icInstance;
    OSType psiSignature = 'Psi ';
    OSStatus error = ICStart( &icInstance, psiSignature );

    ConstStr255Param hint( 0x0 );
    const char* data = m_url.toLocal8Bit().data();
    long length = m_url.length();
    long start( 0 );
    long end( length );

    ICLaunchURL( icInstance, hint, data, length, &start, &end );
    ICStop( icInstance );
    #endif

    #ifdef Q_WS_X11
    QStringList params;
    params.append( QString( QUrl( m_url ).toEncoded() ) );

    QStringList options;
    options = Settings::instance()->browser().split( " " );
    if ( options.size() == 0 )
        options.append( Settings::instance()->browser() );

    for ( int i = 1; i < options.size(); i++ )
    {
        qDebug( QString( "param: " + options.at( i ) ).toLocal8Bit() );
        params.append( "\"" + options.at( i ) + "\"" );
    }

    QProcess::execute( options.at( 0 ), params );
    #endif
}

